@@ -20,15 +20,6 @@
 #ifndef _NTIOCTL_H
 #define _NTIOCTL_H
 
-#define IO_REPARSE_TAG_SYMLINK	     0xA000000C
-#define SYMLINK_FLAG_RELATIVE	     0x00000001
-
-#define IO_REPARSE_TAG_MOUNT_POINT   0xA0000003
-#define IO_REPARSE_TAG_HSM           0xC0000004
-#define IO_REPARSE_TAG_SIS           0x80000007
-#define IO_REPARSE_TAG_DFS	     0x8000000A
-#define IO_REPARSE_TAG_NFS	     0x80000014
-
 /* Sub-types for IO_REPARSE_TAG_NFS */
 #define NFS_SPECFILE_LNK	     0x00000000014B4E4C
 #define NFS_SPECFILE_CHR	     0x0000000000524843
