@@ -994,6 +994,7 @@ static int get_pso_count(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 	struct ldb_result *res = NULL;
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 
+	*pso_count = 0;
 	domain_dn = ldb_get_default_basedn(ldb);
 	psc_dn = ldb_dn_new_fmt(mem_ctx, ldb,
 			        "CN=Password Settings Container,CN=System,%s",
@@ -1007,6 +1008,17 @@ static int get_pso_count(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 				 LDB_SCOPE_ONELEVEL, attrs,
 				 DSDB_FLAG_NEXT_MODULE, parent,
 				 "(objectClass=msDS-PasswordSettings)");
+
+	/*
+	 * Just ignore PSOs if the container doesn't exist. This is a weird
+	 * corner-case where the AD DB was created from a pre-2008 base schema,
+	 * and then the FL was manually upgraded.
+	 */
+	if (ret == LDB_ERR_NO_SUCH_OBJECT) {
+		DBG_NOTICE("No Password Settings Container exists\n");
+		return LDB_SUCCESS;
+	}
+
 	if (ret != LDB_SUCCESS) {
 		return ret;
 	}
@@ -1023,8 +1035,7 @@ static int get_pso_count(struct ldb_module *module, TALLOC_CTX *mem_ctx,
  * The PSO with the lowest precedence is better, otherwise (if the precedence
  * is equal) the PSO with the lower GUID wins.
  */
-static int pso_compare(struct ldb_message **m1, struct ldb_message **m2,
-		       TALLOC_CTX *mem_ctx)
+static int pso_compare(struct ldb_message **m1, struct ldb_message **m2)
 {
 	uint32_t prec1;
 	uint32_t prec2;
@@ -1122,7 +1133,7 @@ static int pso_find_best(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 	}
 
 	/* sort the list so that the best PSO is first */
-	LDB_TYPESAFE_QSORT(res->msgs, res->count, mem_ctx, pso_compare);
+	TYPESAFE_QSORT(res->msgs, res->count, pso_compare);
 
 	if (res->count > 0) {
 		*best_pso = res->msgs[0];
@@ -1388,7 +1399,7 @@ static const struct op_attributes_replace search_sub[] = {
 	{ "tokenGroups", "primaryGroupID", objectSid_attr, construct_token_groups },
 	{ "tokenGroupsNoGCAcceptable", "primaryGroupID", objectSid_attr, construct_token_groups_no_gc},
 	{ "tokenGroupsGlobalAndUniversal", "primaryGroupID", objectSid_attr, construct_global_universal_token_groups },
-	{ "parentGUID", NULL, NULL, construct_parent_guid },
+	{ "parentGUID", "objectGUID", NULL, construct_parent_guid },
 	{ "subSchemaSubEntry", NULL, NULL, construct_subschema_subentry },
 	{ "msDS-isRODC", "objectClass", objectCategory_attr, construct_msds_isrodc },
 	{ "msDS-KeyVersionNumber", "replPropertyMetaData", NULL, construct_msds_keyversionnumber },
